@@ -1,7 +1,7 @@
 /* Analyze file differences for GNU DIFF.
 
    Copyright (C) 1988-1989, 1992-1995, 1998, 2001-2002, 2004, 2006-2007,
-   2009-2013, 2015-2023 Free Software Foundation, Inc.
+   2009-2013, 2015-2025 Free Software Foundation, Inc.
 
    This file is part of GNU DIFF.
 
@@ -19,7 +19,9 @@
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 #include "diff.h"
+
 #include <cmpbuf.h>
+#include <diagnose.h>
 #include <error.h>
 #include <file-type.h>
 #include <xalloc.h>
@@ -30,9 +32,11 @@
 #define OFFSET lin
 #define OFFSET_MAX LIN_MAX
 #define EXTRA_CONTEXT_FIELDS /* none */
-#define NOTE_DELETE(c, xoff) (files[0].changed[files[0].realindexes[xoff]] = 1)
-#define NOTE_INSERT(c, yoff) (files[1].changed[files[1].realindexes[yoff]] = 1)
-#define USE_HEURISTIC 1
+#define NOTE_DELETE(c, x) \
+  (curr.file[0].changed[curr.file[0].realindexes[x]] = true)
+#define NOTE_INSERT(c, y) \
+  (curr.file[1].changed[curr.file[1].realindexes[y]] = true)
+#define USE_HEURISTIC
 #include <diffseq.h>
 
 /* Discard lines from one file that have no matches in the other file.
@@ -50,16 +54,10 @@
 static void
 discard_confusing_lines (struct file_data filevec[])
 {
-  int f;
-  lin i;
-  char *discarded[2];
-  lin *equiv_count[2];
-  lin *p;
-
   /* Allocate our results.  */
-  p = xmalloc ((filevec[0].buffered_lines + filevec[1].buffered_lines)
-               * (2 * sizeof *p));
-  for (f = 0; f < 2; f++)
+  lin *p = xinmalloc (filevec[0].buffered_lines + filevec[1].buffered_lines,
+		      2 * sizeof *p);
+  for (int f = 0; f < 2; f++)
     {
       filevec[f].undiscarded = p;  p += filevec[f].buffered_lines;
       filevec[f].realindexes = p;  p += filevec[f].buffered_lines;
@@ -68,44 +66,43 @@ discard_confusing_lines (struct file_data filevec[])
   /* Set up equiv_count[F][I] as the number of lines in file F
      that fall in equivalence class I.  */
 
-  p = xcalloc (filevec[0].equiv_max, 2 * sizeof *p);
+  p = xicalloc (filevec[0].equiv_max, 2 * sizeof *p);
+  lin *equiv_count[2];
   equiv_count[0] = p;
   equiv_count[1] = p + filevec[0].equiv_max;
 
-  for (i = 0; i < filevec[0].buffered_lines; ++i)
+  for (lin i = 0; i < filevec[0].buffered_lines; i++)
     ++equiv_count[0][filevec[0].equivs[i]];
-  for (i = 0; i < filevec[1].buffered_lines; ++i)
+  for (lin i = 0; i < filevec[1].buffered_lines; i++)
     ++equiv_count[1][filevec[1].equivs[i]];
 
   /* Set up tables of which lines are going to be discarded.  */
 
-  discarded[0] = xzalloc (filevec[0].buffered_lines
-			  + filevec[1].buffered_lines);
+  char *discarded[2];
+  discarded[0] = xizalloc (filevec[0].buffered_lines
+			   + filevec[1].buffered_lines);
   discarded[1] = discarded[0] + filevec[0].buffered_lines;
 
   /* Mark to be discarded each line that matches no line of the other file.
      If a line matches many lines, mark it as provisionally discardable.  */
 
-  for (f = 0; f < 2; f++)
+  for (int f = 0; f < 2; f++)
     {
-      size_t end = filevec[f].buffered_lines;
+      lin end = filevec[f].buffered_lines;
       char *discards = discarded[f];
       lin *counts = equiv_count[1 - f];
       lin *equivs = filevec[f].equivs;
-      size_t many = 5;
-      size_t tem = end / 64;
+      lin many = 5;
 
       /* Multiply MANY by approximate square root of number of lines.
          That is the threshold for provisionally discardable lines.  */
-      while ((tem = tem >> 2) > 0)
-        many *= 2;
+      many <<= end < 64 ? 0 : (floor_log2 (end) >> 1) - 3;
 
-      for (i = 0; i < end; i++)
+      for (lin i = 0; i < end; i++)
         {
-          lin nmatch;
           if (equivs[i] == 0)
             continue;
-          nmatch = counts[equivs[i]];
+          lin nmatch = counts[equivs[i]];
           if (nmatch == 0)
             discards[i] = 1;
           else if (nmatch > many)
@@ -117,12 +114,12 @@ discard_confusing_lines (struct file_data filevec[])
      in a run of discardables, with nonprovisionals at the beginning
      and end.  */
 
-  for (f = 0; f < 2; f++)
+  for (int f = 0; f < 2; f++)
     {
       lin end = filevec[f].buffered_lines;
-      register char *discards = discarded[f];
+      char *discards = discarded[f];
 
-      for (i = 0; i < end; i++)
+      for (lin i = 0; i < end; i++)
         {
           /* Cancel provisional discards not in middle of run of discards.  */
           if (discards[i] == 2)
@@ -130,9 +127,7 @@ discard_confusing_lines (struct file_data filevec[])
           else if (discards[i] != 0)
             {
               /* We have found a nonprovisional discard.  */
-              register lin j;
-              lin length;
-              lin provisional = 0;
+              lin provisional = 0, j;
 
               /* Find end of this run of discardable lines.
                  Count how many are provisionally discardable.  */
@@ -150,11 +145,11 @@ discard_confusing_lines (struct file_data filevec[])
 
               /* Now we have the length of a run of discardable lines
                  whose first and last are not provisional.  */
-              length = j - i;
+              lin length = j - i;
 
               /* If 1/4 of the lines in the run are provisional,
                  cancel discarding of all provisional lines in the run.  */
-              if (provisional * 4 > length)
+	      if (length >> 2 < provisional)
                 {
                   while (j > i)
                     if (discards[--j] == 2)
@@ -162,21 +157,18 @@ discard_confusing_lines (struct file_data filevec[])
                 }
               else
                 {
-                  register lin consec;
-                  lin minimum = 1;
-                  lin tem = length >> 2;
-
                   /* MINIMUM is approximate square root of LENGTH/4.
                      A subrun of two or more provisionals can stand
                      when LENGTH is at least 16.
                      A subrun of 4 or more can stand when LENGTH >= 64.  */
-                  while (0 < (tem >>= 2))
-                    minimum <<= 1;
-                  minimum++;
+		  lin minimum =
+		    (length < 4 ? 2
+		     : ((lin) 1 << ((floor_log2 (length) >> 1) - 1)) + 1);
 
                   /* Cancel any subrun of MINIMUM or more provisionals
                      within the larger run.  */
-                  for (j = 0, consec = 0; j < length; j++)
+                  lin consec = 0;
+                  for (j = 0; j < length; j++)
                     if (discards[i + j] != 2)
                       consec = 0;
                     else if (minimum == ++consec)
@@ -226,19 +218,19 @@ discard_confusing_lines (struct file_data filevec[])
     }
 
   /* Actually discard the lines. */
-  for (f = 0; f < 2; f++)
+  for (int f = 0; f < 2; f++)
     {
       char *discards = discarded[f];
       lin end = filevec[f].buffered_lines;
       lin j = 0;
-      for (i = 0; i < end; ++i)
+      for (lin i = 0; i < end; i++)
         if (minimal || discards[i] == 0)
           {
             filevec[f].undiscarded[j] = filevec[f].equivs[i];
             filevec[f].realindexes[j++] = i;
           }
         else
-          filevec[f].changed[i] = 1;
+          filevec[f].changed[i] = true;
       filevec[f].nondiscarded_lines = j;
     }
 
@@ -259,21 +251,17 @@ discard_confusing_lines (struct file_data filevec[])
 static void
 shift_boundaries (struct file_data filevec[])
 {
-  int f;
-
-  for (f = 0; f < 2; f++)
+  for (int f = 0; f < 2; f++)
     {
-      char *changed = filevec[f].changed;
-      char *other_changed = filevec[1 - f].changed;
+      bool *changed = filevec[f].changed;
+      bool *other_changed = filevec[1 - f].changed;
       lin const *equivs = filevec[f].equivs;
       lin i = 0;
       lin j = 0;
       lin i_end = filevec[f].buffered_lines;
 
-      while (1)
+      while (true)
         {
-          lin runlength, start, corresponding;
-
           /* Scan forwards to find beginning of another run of changes.
              Also keep track of the corresponding point in the other file.  */
 
@@ -287,7 +275,7 @@ shift_boundaries (struct file_data filevec[])
           if (i == i_end)
             break;
 
-          start = i;
+          lin start = i;
 
           /* Find the end of this run of changes.  */
 
@@ -296,6 +284,8 @@ shift_boundaries (struct file_data filevec[])
           while (other_changed[j])
             j++;
 
+          lin runlength, corresponding;
+
           do
             {
               /* Record the length of this run of changes, so that
@@ -308,8 +298,8 @@ shift_boundaries (struct file_data filevec[])
 
               while (start && equivs[start - 1] == equivs[i - 1])
                 {
-                  changed[--start] = 1;
-                  changed[--i] = 0;
+                  changed[--start] = true;
+                  changed[--i] = false;
                   while (changed[start - 1])
                     start--;
                   while (other_changed[--j])
@@ -329,8 +319,8 @@ shift_boundaries (struct file_data filevec[])
 
               while (i != i_end && equivs[start] == equivs[i])
                 {
-                  changed[start++] = 0;
-                  changed[i++] = 1;
+                  changed[start++] = false;
+                  changed[i++] = true;
                   while (changed[i])
                     i++;
                   while (other_changed[++j])
@@ -344,8 +334,8 @@ shift_boundaries (struct file_data filevec[])
 
           while (corresponding < i)
             {
-              changed[--start] = 1;
-              changed[--i] = 0;
+              changed[--start] = true;
+              changed[--i] = false;
               while (other_changed[--j])
                 continue;
             }
@@ -381,9 +371,9 @@ add_change (lin line0, lin line1, lin deleted, lin inserted,
 static struct change *
 build_reverse_script (struct file_data const filevec[])
 {
-  struct change *script = 0;
-  char *changed0 = filevec[0].changed;
-  char *changed1 = filevec[1].changed;
+  struct change *script = nullptr;
+  bool *changed0 = filevec[0].changed;
+  bool *changed1 = filevec[1].changed;
   lin len0 = filevec[0].buffered_lines;
   lin len1 = filevec[1].buffered_lines;
 
@@ -418,9 +408,9 @@ build_reverse_script (struct file_data const filevec[])
 static struct change *
 build_script (struct file_data const filevec[])
 {
-  struct change *script = 0;
-  char *changed0 = filevec[0].changed;
-  char *changed1 = filevec[1].changed;
+  struct change *script = nullptr;
+  bool *changed0 = filevec[0].changed;
+  bool *changed1 = filevec[1].changed;
   lin i0 = filevec[0].buffered_lines, i1 = filevec[1].buffered_lines;
 
   /* Note that changedN[-1] does exist, and is 0.  */
@@ -454,20 +444,16 @@ briefly_report (int changes, struct file_data const filevec[])
     message ((brief
               ? N_("Files %s and %s differ\n")
               : N_("Binary files %s and %s differ\n")),
-             file_label[0] ? file_label[0] : filevec[0].name,
-             file_label[1] ? file_label[1] : filevec[1].name);
+	     file_label[0] ? file_label[0] : squote (0, filevec[0].name),
+	     file_label[1] ? file_label[1] : squote (1, filevec[1].name));
 }
 
 /* Report the differences of two files.  */
 int
 diff_2_files (struct comparison *cmp)
 {
-  int f;
-  struct change *e, *p;
-  struct change *script;
   int changes;
 
-
   /* If we have detected that either file is binary,
      compare the two files as binary.  This can happen
      only when the first chunk is read.
@@ -478,8 +464,8 @@ diff_2_files (struct comparison *cmp)
     {
       /* Files with different lengths must be different.  */
       if (cmp->file[0].stat.st_size != cmp->file[1].stat.st_size
-          && 0 < cmp->file[0].stat.st_size
-          && 0 < cmp->file[1].stat.st_size
+	  && 0 <= cmp->file[0].stat.st_size
+	  && 0 <= cmp->file[1].stat.st_size
           && (cmp->file[0].desc < 0 || S_ISREG (cmp->file[0].stat.st_mode))
           && (cmp->file[1].desc < 0 || S_ISREG (cmp->file[1].stat.st_mode)))
         changes = 1;
@@ -492,20 +478,22 @@ diff_2_files (struct comparison *cmp)
         /* Scan both files, a buffer at a time, looking for a difference.  */
         {
           /* Allocate same-sized buffers for both files.  */
-          size_t lcm_max = PTRDIFF_MAX - 1;
-          size_t buffer_size =
+          idx_t lcm_max = IDX_MAX - 1, blksize[2];
+	  for (int f = 0; f < 2; f++)
+	    if (ST_BLKSIZE (cmp->file[f].stat) < 0
+		|| ckd_add (&blksize[f], ST_BLKSIZE (cmp->file[f].stat), 0))
+	      blksize[f] = 0;
+          idx_t buffer_size =
             buffer_lcm (sizeof (word),
-                        buffer_lcm (STAT_BLOCKSIZE (cmp->file[0].stat),
-                                    STAT_BLOCKSIZE (cmp->file[1].stat),
-                                    lcm_max),
+                        buffer_lcm (blksize[0], blksize[1], lcm_max),
                         lcm_max);
-          for (f = 0; f < 2; f++)
-            cmp->file[f].buffer = xrealloc (cmp->file[f].buffer, buffer_size);
+          for (int f = 0; f < 2; f++)
+            cmp->file[f].buffer = xirealloc (cmp->file[f].buffer, buffer_size);
 
           for (;; cmp->file[0].buffered = cmp->file[1].buffered = 0)
             {
               /* Read a buffer's worth from both files.  */
-              for (f = 0; f < 2; f++)
+              for (int f = 0; f < 2; f++)
                 if (0 <= cmp->file[f].desc)
                   file_block_read (&cmp->file[f],
                                    buffer_size - cmp->file[f].buffered);
@@ -533,17 +521,13 @@ diff_2_files (struct comparison *cmp)
     }
   else
     {
-      struct context ctxt;
-      lin diags;
-      lin too_expensive;
-
       /* Allocate vectors for the results of comparison:
          a flag for each line of each file, saying whether that line
          is an insertion or deletion.
-         Allocate an extra element, always 0, at each end of each vector.  */
+         Allocate an extra false element at each end of each vector.  */
 
-      size_t s = cmp->file[0].buffered_lines + cmp->file[1].buffered_lines + 4;
-      char *flag_space = xzalloc (s);
+      bool *flag_space = xizalloc (cmp->file[0].buffered_lines
+				   + cmp->file[1].buffered_lines + 4);
       cmp->file[0].changed = flag_space + 1;
       cmp->file[1].changed = flag_space + cmp->file[0].buffered_lines + 3;
 
@@ -556,11 +540,12 @@ diff_2_files (struct comparison *cmp)
       /* Now do the main comparison algorithm, considering just the
          undiscarded lines.  */
 
+      struct context ctxt;
       ctxt.xvec = cmp->file[0].undiscarded;
       ctxt.yvec = cmp->file[1].undiscarded;
-      diags = (cmp->file[0].nondiscarded_lines
-               + cmp->file[1].nondiscarded_lines + 3);
-      ctxt.fdiag = xmalloc (diags * (2 * sizeof *ctxt.fdiag));
+      lin diags = (cmp->file[0].nondiscarded_lines
+		   + cmp->file[1].nondiscarded_lines + 3);
+      ctxt.fdiag = xinmalloc (diags, 2 * sizeof *ctxt.fdiag);
       ctxt.bdiag = ctxt.fdiag + diags;
       ctxt.fdiag += cmp->file[1].nondiscarded_lines + 1;
       ctxt.bdiag += cmp->file[1].nondiscarded_lines + 1;
@@ -570,13 +555,10 @@ diff_2_files (struct comparison *cmp)
       /* Set TOO_EXPENSIVE to be the approximate square root of the
          input size, bounded below by 4096.  4096 seems to be good for
          circa-2016 CPUs; see Bug#16848 and Bug#24715.  */
-      too_expensive = 1;
-      for (;  diags != 0;  diags >>= 2)
-        too_expensive <<= 1;
+      lin too_expensive = (lin) 1 << ((floor_log2 (diags) >> 1) + 1);
       ctxt.too_expensive = MAX (4096, too_expensive);
 
-      files[0] = cmp->file[0];
-      files[1] = cmp->file[1];
+      curr = *cmp;
 
       compareseq (0, cmp->file[0].nondiscarded_lines,
                   0, cmp->file[1].nondiscarded_lines, minimal, &ctxt);
@@ -590,27 +572,22 @@ diff_2_files (struct comparison *cmp)
 
       /* Get the results of comparison in the form of a chain
          of 'struct change's -- an edit script.  */
-
-      if (output_style == OUTPUT_ED)
-        script = build_reverse_script (cmp->file);
-      else
-        script = build_script (cmp->file);
+      struct change *script = ((output_style == OUTPUT_ED
+				? build_reverse_script
+				: build_script)
+			       (cmp->file));
 
       /* Set CHANGES if we had any diffs.
          If some changes are ignored, we must scan the script to decide.  */
       if (ignore_blank_lines || ignore_regexp.fastmap)
         {
-          struct change *next = script;
           changes = 0;
 
-          while (next && changes == 0)
+          for (struct change *next = script; next && changes == 0; )
             {
-              struct change *this, *end;
-              lin first0, last0, first1, last1;
-
               /* Find a set of changes that belong together.  */
-              this = next;
-              end = find_change (next);
+              struct change *this = next;
+              struct change *end = find_change (next);
 
               /* Disconnect them from the rest of the changes, making them
                  a hunk, and remember the rest for next iteration.  */
@@ -618,6 +595,7 @@ diff_2_files (struct comparison *cmp)
               end->link = 0;
 
               /* Determine whether this hunk is really a difference.  */
+              lin first0, last0, first1, last1;
               if (analyze_hunk (this, &first0, &last0, &first1, &last1))
                 changes = 1;
 
@@ -638,7 +616,7 @@ diff_2_files (struct comparison *cmp)
                  to be used if and when we have some output to print.  */
               setup_output (file_label[0] ? file_label[0] : cmp->file[0].name,
                             file_label[1] ? file_label[1] : cmp->file[1].name,
-                            cmp->parent != 0);
+			    cmp->parent != &noparent);
 
               switch (output_style)
                 {
@@ -675,7 +653,7 @@ diff_2_files (struct comparison *cmp)
                   break;
 
                 default:
-                  abort ();
+                  unreachable ();
                 }
 
               finish_output ();
@@ -686,24 +664,26 @@ diff_2_files (struct comparison *cmp)
 
       free (flag_space);
 
-      for (f = 0; f < 2; f++)
+      for (int f = 0; f < 2; f++)
         {
           free (cmp->file[f].equivs);
           free (cmp->file[f].linbuf + cmp->file[f].linbuf_base);
         }
 
-      for (e = script; e; e = p)
+      for (struct change *e = script; e; )
         {
-          p = e->link;
+          struct change *p = e->link;
           free (e);
+	  e = p;
         }
 
       if (! robust_output_style (output_style))
-        for (f = 0; f < 2; ++f)
+        for (int f = 0; f < 2; f++)
           if (cmp->file[f].missing_newline)
             {
               error (0, 0, "%s: %s\n",
-                     file_label[f] ? file_label[f] : cmp->file[f].name,
+		     (file_label[f] ? file_label[f]
+		      : squote (0, cmp->file[f].name)),
                      _("No newline at end of file"));
               changes = 2;
             }
