@@ -1,6 +1,6 @@
 /* Normal-format output routines for GNU DIFF.
 
-   Copyright (C) 1988-1989, 1993, 1995, 1998, 2001, 2006, 2009-2013, 2015-2023
+   Copyright (C) 1988-1989, 1993, 1995, 1998, 2001, 2006, 2009-2013, 2015-2025
    Free Software Foundation, Inc.
 
    This file is part of GNU DIFF.
@@ -38,10 +38,8 @@ print_normal_script (struct change *script)
 static void
 print_normal_hunk (struct change *hunk)
 {
-  lin first0, last0, first1, last1;
-  register lin i;
-
   /* Determine range of line numbers involved in each file.  */
+  lin first0, last0, first1, last1;
   enum changes changes = analyze_hunk (hunk, &first0, &last0, &first1, &last1);
   if (!changes)
     return;
@@ -50,21 +48,21 @@ print_normal_hunk (struct change *hunk)
 
   /* Print out the line number header for this hunk */
   set_color_context (LINE_NUMBER_CONTEXT);
-  print_number_range (',', &files[0], first0, last0);
+  print_number_range (',', &curr.file[0], first0, last0);
   fputc (change_letter[changes], outfile);
-  print_number_range (',', &files[1], first1, last1);
+  print_number_range (',', &curr.file[1], first1, last1);
   set_color_context (RESET_CONTEXT);
   fputc ('\n', outfile);
 
   /* Print the lines that the first file has.  */
   if (changes & OLD)
     {
-      for (i = first0; i <= last0; i++)
+      for (lin i = first0; i <= last0; i++)
         {
           set_color_context (DELETE_CONTEXT);
-          print_1_line_nl ("<", &files[0].linbuf[i], true);
+	  print_1_line_nl ("<", &curr.file[0].linbuf[i], true);
           set_color_context (RESET_CONTEXT);
-          if (files[0].linbuf[i + 1][-1] == '\n')
+	  if (curr.file[0].linbuf[i + 1][-1] == '\n')
             putc ('\n', outfile);
         }
     }
@@ -75,12 +73,12 @@ print_normal_hunk (struct change *hunk)
   /* Print the lines that the second file has.  */
   if (changes & NEW)
     {
-      for (i = first1; i <= last1; i++)
+      for (lin i = first1; i <= last1; i++)
         {
           set_color_context (ADD_CONTEXT);
-          print_1_line_nl (">", &files[1].linbuf[i], true);
+	  print_1_line_nl (">", &curr.file[1].linbuf[i], true);
           set_color_context (RESET_CONTEXT);
-          if (files[1].linbuf[i + 1][-1] == '\n')
+	  if (curr.file[1].linbuf[i + 1][-1] == '\n')
             putc ('\n', outfile);
         }
     }
